#! /bin/bash

#export ROOT_PATH=/usr/share/bili_tools
export ROOT_PATH=`pwd`
export IO_PATH=$ROOT_PATH/io
export SCHED_PATH=$ROOT_PATH/sched
export MM_PATH=$ROOT_PATH/mm
export FLAME_PATH=$ROOT_PATH/common
export NCCL_PATH=$ROOT_PATH/nccl_tools
export ARCH_PATH=$ROOT_PATH/arch
export NET_PATH=$ROOT_PATH/net

opt=$@
SUBSYS=$1

usage()
{
    echo "Usage: $0 \$subsys \$ARGS"
    echo "The following are all subsystems:"
    echo -e "  arch\t\tQuick tools for architecture"
    echo -e "  io\t\tget info from io subsystem"
    echo -e "  mm\t\tget info from memory subsystem"
    echo -e "  sched\t\tget info from scheduler subsystem"
    echo -e "  flame\t\tgenerate flamegraph"
    echo -e "  nccl\t\tQuick tools of NVIDIA GPU NCCL"
    echo -e "  net\t\tget info from net subsystem"
}

do_io_cmd()
{
    $IO_PATH/iostat_kernel.py $opt
}

do_sched_cmd()
{
    $SCHED_PATH/sched_test $opt
}

do_mm_cmd()
{
    $MM_PATH/mem_bilitools $opt
}

do_flame_cmd()
{
    $FLAME_PATH/flamegraph.sh $opt
}

do_nccl_cmd()
{
    $NCCL_PATH/nccl_tool.sh $opt
}

do_net_cmd()
{
    $NET_PATH/tcp_conn_top $opt
}

do_arch_cmd()
{
    $ARCH_PATH/arch_tool.sh $opt
}

if [ $# -lt 1 ];then
    usage
    exit 1
fi

case "$SUBSYS" in
	io)
		opt=${opt:2}
		do_io_cmd
		;;
	mm)
		opt=${opt:2}
		do_mm_cmd
		;;
	sched)
		opt=${opt:5}
		do_sched_cmd
		;;
	flame)
		opt=${opt:5}
		do_flame_cmd
		;;
	nccl)
		opt=${opt:4}
		do_nccl_cmd
		;;
	arch)
		opt=${opt:4}
		do_arch_cmd
		;;
	net)
		opt=${opt:4}
		do_net_cmd
		;;
	*)
		echo "Unknown subsystem"
		usage
		exit 1
		;;
esac

exit 0
