import os
import datetime
import torch
import torch.distributed as dist

def init_workers_nccl_file():
    rank = int(os.environ['RANK'])
    n_ranks = int(os.environ['WORLD_SIZE'])
    ranks_per_node = int(os.environ['LOCAL_SIZE'])
    iters = int(os.environ['NR_ITERATION'])
    dist.init_process_group(backend='nccl', world_size=n_ranks, rank=rank)
    return rank, n_ranks, ranks_per_node, iters

# Print pytorch version
print('Pytorch version', torch.__version__)

# Configuration
dtype = torch.float32

# Initialize MPI
rank, n_ranks, ranks_per_node, iters = init_workers_nccl_file()
local_rank = rank % ranks_per_node

shape = 2**20
num_tensors = 2**8
size_bytes = num_tensors * shape * n_ranks

# Allocate a small tensor on every gpu from every rank.
# This is an attempt to force creation of all device contexts.
#for i in range(ranks_per_node):
#    _ = torch.randn(1).to(torch.device('cuda', i))

# Select our gpu
device = torch.device('cuda', local_rank)
torch.cuda.set_device(device)
print('local_rank', local_rank,  'rank', rank, 'total', n_ranks, 'size_mb', size_bytes // 1e6)

#print('ranks_per_node', ranks_per_node,
#        'size_mb', size_bytes // 1e6,
#        'stream', torch.cuda.current_stream(),
#        'count', torch.cuda.device_count())

x = [torch.randn(shape, dtype=dtype).to(device) for i in range(num_tensors)]
start = torch.cuda.Event(enable_timing=True)
end = torch.cuda.Event(enable_timing=True)
start.record()
for i in range(iters):
    # Do an all-reduce
    handles = [dist.all_reduce(data, op=torch.distributed.ReduceOp.AVG, async_op=True) for data in x]
    _ = [h.wait() for h in handles]
    if i and i % 10 == 0:
        print('rank', rank, 'iteration', i)
end.record()
torch.cuda.synchronize(device)

print('rank', rank, 'duration', start.elapsed_time(end), 'bandwidth', iters * size_bytes / start.elapsed_time(end), 'KB/s')
